package com.zhentao.moment.service.impl;

import com.aliyun.oss.OSSClient;
import com.aliyun.oss.model.ObjectMetadata;
import com.aliyun.oss.model.PutObjectRequest;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhentao.moment.config.OssConfig;
import com.zhentao.moment.config.OssProperties;
import com.zhentao.moment.domain.UserMoments;
import com.zhentao.moment.dto.MonmentDto;
import com.zhentao.moment.enums.ContentTypeEnum;
import com.zhentao.moment.service.UserMomentsService;
import com.zhentao.moment.mapper.UserMomentsMapper;
import com.zhentao.tool.TokenUtils;
import com.zhentao.user.domain.UserLogin;
import com.zhentao.user.mapper.UserLoginMapper;
import com.zhentao.utils.SensitiveWordFilter;
import com.zhentao.utils.SnowflakeUtil;
import com.zhentao.vo.Result;
import io.jsonwebtoken.Claims;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* @author 86159
* @description 针对表【user_moments(朋友圈动态表)】的数据库操作Service实现
* @createDate 2025-06-04 11:56:59
*/
@Service
public class UserMomentsServiceImpl extends ServiceImpl<UserMomentsMapper, UserMoments>
    implements UserMomentsService{
    @Autowired
    private UserMomentsMapper userMomentsMapper;
    @Autowired
    private UserLoginMapper userLoginMapper;
    @Autowired

    private OSSClient ossClient;
    @Autowired
    private OssProperties ossProperties;
    @Override
    public Result sendMonment(String token,MonmentDto monmentDto) {
        if (monmentDto.getContent()==null){
            return Result.error(400,"请输入内容");
        }
        if (monmentDto.getContentType()==null){
            return Result.error(400,"内容类型不能为空");
        }
        try {
            UserMoments userMoments=new UserMoments();
            userMoments.setMomentId(SnowflakeUtil.nextId());
            String userId = TokenUtils.getUserIdFromToken(token);
            userMoments.setUserId(Long.valueOf(userId));
            String filter = SensitiveWordFilter.filter(monmentDto.getContent());
            userMoments.setContent(filter);
            Integer typeCode = monmentDto.getContentType();
            ContentTypeEnum type = ContentTypeEnum.getByCode(typeCode);
            if (monmentDto.getFiles()!=null && !monmentDto.getFiles().isEmpty()){
                if (type!=ContentTypeEnum.IMAGE){
                    return Result.error(400,"只有图片类型支持上传多张");
                }
            switch (type){
                case IMAGE:
                    for (MultipartFile file : monmentDto.getFiles()){
                        if (file.isEmpty()){
                            return Result.error(400,"图片不能为空");
                        }
                        validateImageContent(file);
                    }
                    List<String> imageUrls = new ArrayList<>();
                    List<String> imageNames = new ArrayList<>();
                    Long totalSize = 0L;
                    String imageDir = ossProperties.getUpload().getImage().getDir();
                    for (MultipartFile file : monmentDto.getFiles()){
                        String imageFileName = userId + "/" + System.currentTimeMillis() +
                                getFileExtension(file.getOriginalFilename());
                        String ossFilePath = imageDir + imageFileName;
//                        上传OSS
                        String url = uploadToOss(file, ossFilePath);
                        imageUrls.add(url);
                        imageNames.add(file.getOriginalFilename());
                        totalSize+=file.getSize();
                    }
                    String urlJson = "[" + imageUrls.stream()
                            .map(url -> "\"" + url.replace("\"", "\\\"") + "\"") // 确保URL中的双引号被转义
                            .collect(Collectors.joining(",")) + "]";
                    userMoments.setUrl(urlJson);
                    String nameJson = "[" + imageNames.stream()
                            .map(name -> "\"" + name.replace("\"", "\\\"") + "\"")
                            .collect(Collectors.joining(",")) + "]";
                    userMoments.setFilename(nameJson);
                    userMoments.setFilesize(totalSize);

                    break;
                case VIDEO:
                    MultipartFile videoFile = monmentDto.getFiles().get(0);
                    if (videoFile.isEmpty()){
                        return Result.error(400,"视频不能为空");
                    }
                    validateVideoContent(videoFile);
                    String videoDir = ossProperties.getUpload().getVideo().getDir();
                    String videoFileName = userId + "/" + System.currentTimeMillis() +
                            getFileExtension(videoFile.getOriginalFilename());
                    String ossFilePath = videoDir + videoFileName;
                    String url = uploadToOss(videoFile, ossFilePath);

                    userMoments.setUrl(url);

                    userMoments.setFilename(videoFile.getOriginalFilename());
                    userMoments.setFilesize(videoFile.getSize());
                    break;
                default:
                return Result.error(400,"内容类型错误");
            }
//            上传文件到OSS

            }else if (type!=ContentTypeEnum.TEXT){
                return Result.ERR(null,"类型上传不正确");
            }
////            userMoments.setContentType();
//            userMoments.setContent(monmentDto.getContent());
            userMoments.setVisibility(monmentDto.getVisible());
            userMomentsMapper.insert(userMoments);
            Map<String,Object> map=new HashMap<>();
            map.put("monmentId",userMoments.getMomentId());
            map.put("ossFilePath",userMoments.getUrl());
            return Result.OK(map,"发送成功");
        }catch (Exception e){
            e.printStackTrace();
            return Result.error(500,"发送失败");
        }

    }
// 删除朋友圈
    @Override
    public Result deleteMonment(MonmentDto monmentDto) {
        return null;
    }
//查询个人信息
    @Override
    public Result userinfo(String token) {
        String userId = TokenUtils.getUserIdFromToken(token);
        if (userId==null){
            return Result.error(400,"没有userid");
        }
        UserLogin userLogin = userLoginMapper.selectById(userId);
        return Result.OK(userLogin,"查询成功");
    }

    @Override
    public List<UserMoments> getMyMonment(Long userId) {
        QueryWrapper<UserMoments> queryWrapper=new QueryWrapper<>();
        queryWrapper.eq("user_id",userId);
        List<UserMoments> userMoments = userMomentsMapper.selectList(queryWrapper);
        return userMoments;
    }

    //    **
//            * 上传文件到OSS
//     */
    private String uploadToOss(MultipartFile file, String ossFilePath) throws IOException {
        String bucketName = ossProperties.getBucketName();

        // 创建PutObject请求
        PutObjectRequest putObjectRequest = new PutObjectRequest(bucketName, ossFilePath,
                file.getInputStream(), null);

        // 设置对象元信息
        ObjectMetadata metadata = new ObjectMetadata();
        metadata.setContentType(file.getContentType());
        metadata.setContentLength(file.getSize());
        putObjectRequest.setMetadata(metadata);

        // 上传文件
        ossClient.putObject(putObjectRequest);

        // 构建访问URL
        String endpoint = ossProperties.getEndpoint();
        String cleanEndpoint = endpoint.replaceAll("^https://", "");
        String domain = "https://" + bucketName + "." + cleanEndpoint;
        return domain + "/" + ossFilePath;
    }

    /**
     * 获取文件扩展名
     */
    private String getFileExtension(String fileName) {
        if (fileName == null || !fileName.contains(".")) {
            return "";
        }
        return fileName.substring(fileName.lastIndexOf("."));
    }

    /**
     * 验证图片内容
     */
    private void validateImageContent(MultipartFile file) throws Exception {
        if (file == null || file.isEmpty()) {
            throw new Exception("图片不能为空");
        }

        String contentType = file.getContentType();
        if (!contentType.startsWith("image/")) {
            throw new Exception("请上传图片格式文件");
        }

        // 验证图片大小（例如限制为10MB）
        if (file.getSize() > ossProperties.getUpload().getImage().getMaxSize()) {
            throw new Exception("图片大小不能超过" +
                    (ossProperties.getUpload().getImage().getMaxSize() / (1024 * 1024)) + "MB");
        }

        // 可以添加更多图片验证逻辑，如图片尺寸、格式等
    }

    /**
     * 验证视频内容
     */
    private void validateVideoContent(MultipartFile file) throws Exception {
        if (file == null || file.isEmpty()) {
            throw new Exception("视频不能为空");
        }

        String contentType = file.getContentType();
        if (!contentType.startsWith("video/")) {
            throw new Exception("请上传视频格式文件");
        }

        // 验证视频大小（例如限制为50MB）
        if (file.getSize() > ossProperties.getUpload().getVideo().getMaxSize()) {
            throw new Exception("视频大小不能超过" +
                    (ossProperties.getUpload().getVideo().getMaxSize() / (1024 * 1024)) + "MB");
        }

        // 可以添加更多视频验证逻辑，如视频时长、分辨率等
    }
}




